--[[	PhantomPlates CastBar Text Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);
if AddOn.IsVanillaClient then return; end--	TBC+ Only

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_AbsCeil=AddOn.Utilities.AbsCeil;

----------------------------------
--[[	Number Formatting	]]
----------------------------------
local FormatTime; do--	function FormatTime(val)
	local DayFmt,HrFmt,MinFmt,SecFmt=
		DAY_ONELETTER_ABBR:gsub("%s*%%d%s*","%%0.0f")
		,HOUR_ONELETTER_ABBR:gsub("%s*%%d%s*","%%0.0f")
		,MINUTE_ONELETTER_ABBR:gsub("%s*%%d%s*","%%0.0f")
		,SECOND_ONELETTER_ABBR:gsub("%s*%%d%s*","%%0.1f");

	function FormatTime(val)
		if val>=86400 then return DayFmt:format(AddOn_Utilities_AbsCeil(val/86400));
		elseif val>=3600 then return HrFmt:format(AddOn_Utilities_AbsCeil(val/3600));
		elseif val>=60 then return MinFmt:format(AddOn_Utilities_AbsCeil(val/60));
		else return SecFmt:format(val); end
	end
end

------------------
--[[	Hooks	]]
------------------
local TextObjects={};

local NamePlateCastBar_OnUpdate; do--	function NamePlateCastBar_OnUpdate(self,elapsed)
	local DiffHoldTimer,DiffFadeTimer=0.5,0.5;
	local DiffFullTimer=DiffHoldTimer+DiffFadeTimer;

	local CastTracking={};
	function NamePlateCastBar_OnUpdate(self,elapsed)
		if self:IsForbidden() then return; end
		local parent=self:GetParent();
		if not TextObjects[parent] then return; end

		local CastTimeEnabled=AddOn.Options.NamePlateText_ShowCastTime;
		local CastDiffEnabled=AddOn.Options.NamePlateText_ShowCastDiff;
		if not (CastTimeEnabled or CastDiffEnabled) then return; end

		local timetext,difftext=unpack(TextObjects[parent])

		local track=CastTracking[self];
		if not track then track={}; CastTracking[self]=track; end
		if track.DiffTimer then--	Tick fade timer
			if track.DiffTimer<=elapsed then track.DiffTimer=nil;
			else track.DiffTimer=track.DiffTimer-elapsed; end
		end

		local diff=0;
		if self.casting or self.channeling then
			local cur,max=self.value,self.maxValue;
			local castid=(self.casting and self.castID or select(7,UnitChannelInfo(self.unit)));--	Channeling doesn't have a CastID, using SpellID instead

			if castid==track.CastID then
				diff,track.Value=cur-track.Value+(self.casting and -elapsed or elapsed),cur;
				if diff<0 then track.DiffTimer=DiffFullTimer; end--	Reset fade timer to full
			else track.CastID,track.Value=castid,cur; end

			if CastTimeEnabled then timetext:SetFormattedText("%s / %s",FormatTime(cur),FormatTime(max)); end
			if CastDiffEnabled and diff<0 then difftext:SetText(FormatTime(diff)); end
		else track.CastID=nil; end--	Not casting

		if CastDiffEnabled then difftext:SetAlpha(math.min((track.DiffTimer or 0)/DiffFadeTimer,1)); end
	end
--	hooksecurefunc("CastingBarFrame_OnUpdate",NamePlateCastBar_OnUpdate);
end

AddOn.OnNamePlateCreated:AddCallback(function(_,base)
	local unitframe=base.UnitFrame;
	if not (unitframe:IsForbidden() or TextObjects[unitframe]) then
		local castremain=unitframe.CastBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
		AddOn.Utilities.ModifyFont(castremain,nil,nil,"OUTLINE");
		castremain:SetJustifyH("LEFT");
		castremain:SetPoint("LEFT",unitframe.CastBar,"RIGHT",4,2);

		local castdiff=unitframe.CastBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
		AddOn.Utilities.ModifyFont(castdiff,nil,nil,"OUTLINE");
		castdiff:SetJustifyH("RIGHT");
		castdiff:SetPoint("RIGHT",0,2);
		castdiff:SetTextColor(1,0,0);

		unitframe.CastBar.Text:ClearAllPoints();
		unitframe.CastBar.Text:SetPoint("LEFT",0,2);
		unitframe.CastBar.Text:SetPoint("RIGHT",0,2);
		unitframe.CastBar.Text.Hide=nop;

--		Hooking CastingBarFrame_OnUpdate() is possible, but would create a race condition where it'll work on some frames and not others
		unitframe.CastBar:HookScript("OnUpdate",NamePlateCastBar_OnUpdate);

		if not AddOn.Options.NamePlateText_ShowCastTime then castremain:Hide(); end
		if not AddOn.Options.NamePlateText_ShowCastDiff then castdiff:Hide(); end
		TextObjects[unitframe]={castremain,castdiff};
	end
end);

----------------------------------
--[[	Options Defaults	]]
----------------------------------
AddOn.Options.NamePlateText_ShowCastTime=true;
AddOn.Options.NamePlateText_ShowCastDiff=true;

--------------------------
--[[	Options Layout	]]
--------------------------
local SubPanel=AddOn.OptionsPanel:CreateSubPanel(1);
SubPanel:SetHeight(56);

local ShowCastTimeButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
ShowCastTimeButton:SetPoint("TOPLEFT",4,0);
ShowCastTimeButton.text:SetText(AddOn.Localization.OptionsSetting_CastBarText_ShowCastTime);

local ShowCastDiffButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
ShowCastDiffButton:SetPoint("TOPLEFT",4,-24);
ShowCastDiffButton.text:SetText(AddOn.Localization.OptionsSetting_CastBarText_ShowCastDiff);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.OptionsPanel:RegisterControl("NamePlateText_ShowCastTime",ShowCastTimeButton
	,nil
	,ShowCastTimeButton.SetChecked
	,ShowCastTimeButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:RegisterControl("NamePlateText_ShowCastDiff",ShowCastDiffButton
	,nil
	,ShowCastDiffButton.SetChecked
	,ShowCastDiffButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:HookOption("NamePlateText_ShowCastTime",function(_,_,val)
	for unitframe,text in pairs(TextObjects) do text[1]:SetShown(val); end
end);

AddOn.OptionsPanel:HookOption("NamePlateText_ShowCastDiff",function(_,_,val)
	for unitframe,text in pairs(TextObjects) do text[2]:SetShown(val); end
end);
